@@ -1,35 +1,5 @@
 var indexSectionsWithContent = {
-   "General": {
-      "Symbols": false,
-      "Numbers": true,
-      "A": true,
-      "B": false,
-      "C": true,
-      "D": false,
-      "E": false,
-      "F": true,
-      "G": true,
-      "H": false,
-      "I": false,
-      "J": false,
-      "K": false,
-      "L": true,
-      "M": true,
-      "N": false,
-      "O": false,
-      "P": false,
-      "Q": false,
-      "R": false,
-      "S": false,
-      "T": true,
-      "U": false,
-      "V": false,
-      "W": false,
-      "X": false,
-      "Y": false,
-      "Z": false
-      },
-   "Constants": {
+   "Functions": {
       "Symbols": false,
       "Numbers": false,
       "A": false,
@@ -60,6 +30,36 @@ var indexSectionsWithContent = {
       "Z": false
       },
    "Files": {
+      "Symbols": false,
+      "Numbers": false,
+      "A": true,
+      "B": true,
+      "C": true,
+      "D": false,
+      "E": false,
+      "F": true,
+      "G": true,
+      "H": true,
+      "I": true,
+      "J": true,
+      "K": false,
+      "L": true,
+      "M": true,
+      "N": false,
+      "O": false,
+      "P": true,
+      "Q": false,
+      "R": false,
+      "S": true,
+      "T": true,
+      "U": false,
+      "V": false,
+      "W": true,
+      "X": true,
+      "Y": false,
+      "Z": false
+      },
+   "Macros": {
       "Symbols": false,
       "Numbers": false,
       "A": false,
@@ -89,6 +89,36 @@ var indexSectionsWithContent = {
       "Y": false,
       "Z": false
       },
+   "General": {
+      "Symbols": false,
+      "Numbers": true,
+      "A": true,
+      "B": true,
+      "C": true,
+      "D": false,
+      "E": true,
+      "F": true,
+      "G": true,
+      "H": true,
+      "I": true,
+      "J": true,
+      "K": false,
+      "L": true,
+      "M": true,
+      "N": false,
+      "O": true,
+      "P": true,
+      "Q": false,
+      "R": false,
+      "S": true,
+      "T": true,
+      "U": false,
+      "V": false,
+      "W": true,
+      "X": true,
+      "Y": false,
+      "Z": false
+      },
    "Types": {
       "Symbols": false,
       "Numbers": false,
@@ -119,7 +149,7 @@ var indexSectionsWithContent = {
       "Y": false,
       "Z": false
       },
-   "Functions": {
+   "Constants": {
       "Symbols": false,
       "Numbers": false,
       "A": false,
@@ -140,7 +170,7 @@ var indexSectionsWithContent = {
       "P": false,
       "Q": false,
       "R": false,
-      "S": false,
+      "S": true,
       "T": false,
       "U": false,
       "V": false,
